﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace QuantRiskLib
{
    public static class DateTimeExtensions
    {
        public static bool IsWeekend(this DateTime date)
        {
            return (date.DayOfWeek == DayOfWeek.Saturday || date.DayOfWeek == DayOfWeek.Sunday);
        }

        public static DateTime AddBusinessDays(this DateTime date, int days)
        {
            var sign = Math.Sign(days);
            var unsignedDays = Math.Abs(days);
            for (var i = 0; i < unsignedDays; i++)
            {
                do
                {
                    date = date.AddDays(sign);
                }
                while (date.IsWeekend());
            }
            return date;
        }

        public static List<DateTime> To(this DateTime startDate, DateTime endDate, bool includeWeekends)
        {
            if (startDate > endDate)
                throw new ArgumentException("startDate should be less than or equal to endDate");
            if (startDate.Date == endDate.Date)
                return new List<DateTime> { startDate };

            List<DateTime> dateList = new List<DateTime>();
            if (includeWeekends)
            {
                for (DateTime dt = startDate; dt <= endDate; dt = dt.AddDays(1))
                    dateList.Add(dt);
            }
            else
            {
                for (DateTime dt = startDate; dt <= endDate; dt = dt.AddBusinessDays(1))
                    dateList.Add(dt);
            }
            return dateList;
        }

        public enum MissingDataType
        {
            Price, Return
        }

        public static Dictionary<DateTime, double> AddMissingBusinessDays(Dictionary<DateTime, double> inDictionary, MissingDataType missingDataType)
        {
            Dictionary<DateTime, double> outDict = new Dictionary<DateTime, double>();

            DateTime minDate = inDictionary.Keys.Min();
            DateTime maxDate = inDictionary.Keys.Max();
            double lastPrice = double.NaN;
            for (DateTime dt = minDate; dt <= maxDate; dt = dt.AddBusinessDays(1))
            {
                double value;
                if (inDictionary.TryGetValue(dt, out value))
                {
                    outDict.Add(dt, value);
                    lastPrice = value;
                }
                else
                {
                    if (missingDataType == MissingDataType.Price)
                        outDict.Add(dt, lastPrice);
                    else
                        outDict.Add(dt, 0.0);
                }
            }

            return outDict;
        }
    }
}
